/* 
Quicksort implementation for OFP v1.5 by Faguss (ofp-faguss.com)
Usage: 
	call preProcessFile "quicksort.sqf"
	[<array with numbers>, <any array>, <...>] call quicksort
	[<array with strings>, <any array>, <...>, <optional arguments"] call quicksort	
Input:
	Array
Output:
	Nothing (modifies input array)
Example:
	[[3,2,1], ["C","B","A"]] call quicksort
	[["C","B","A"], [3,2,1], "casesensitive:true natural:true"] call quicksort
	
Input array must contain at least one sub-array that contains either numbers or strings
Optionally you can pass other arrays and they will be sorted in the same way as the first one. 
They may contain any data type but must be the same size as the first array.
When sorting strings then last argument in the input may be a string which contains options for the string comparison:
	- caseSensitive:true - enables case sensitivity
	- reverseCase:true - puts capital letters before lowercase; requires case sensitivity
	- natural:true - sorts by value of the number in the string
*/

// Wrapper function
quicksort = {
	private ["_item", "_options"];
	
	if (count _this > 0) then {
		if (count (_this select 0) > 0) then {
			_item = (_this select 0) select 0;

			// If the first item in the first array is a string then alphabetize
			if (_item in [Format["%1",_item]]) then {
				_options = "";
				
				// Check if input contains options
				if (count _this > 1) then {
					_item = _this select (count _this-1);
					
					if (_item in [_item]) then {
						_options = _item;
						_this resize (count _this-1);
					};
				};
				
				[_this, 0, count (_this select 0)-1, _options] call quicksortABC_loop
			} else {
				[_this, 0, count (_this select 0)-1] call quicksort_loop
			};
		};
	};
};

// Number sorting
quicksort_loop = {
	private ["_input", "_to_sort", "_lo", "_hi", "_i", "_j", "_mid", "_temp"];
	
	_input   = _this select 0;
	_to_sort = _input select 0;
	_lo      = _this select 1;
	_hi      = _this select 2;
	_i       = _lo;
	_j       = _hi;

	// Find pivot
	_mid = (_lo + _hi) / 2;
	_mid = _mid - (_mid mod 1);
	_mid = _to_sort select _mid;

	// Compare
	while {_i <= _j} do {	
		while {(_to_sort select _i) < _mid} do {_i=_i+1};
		while {(_to_sort select _j) > _mid} do {_j=_j-1};
		
		// Replace values in all passed arrays
		if (_i <= _j) then {
			{
				_temp = _x select _i;
				_x set [_i, _x select _j];
				_x set [_j, _temp];
			} forEach _input;
			
			_i = _i + 1;
			_j = _j - 1;
		}
	};

	// Recurrence
	if (_lo < _j) then {[_input, _lo, _j] call quicksort_loop};
	if (_i < _hi) then {[_input, _i, _hi] call quicksort_loop};
};

// String sorting
quicksortABC_loop = {
	private ["_input", "_to_sort", "_lo", "_hi", "_i", "_j", "_options", "_mid", "_temp"];
	
	_input   = _this select 0;
	_to_sort = _input select 0;
	_lo      = _this select 1;
	_hi      = _this select 2;
	_i       = _lo;
	_j       = _hi;
	_options = _this select 3;

	// Find pivot
	_mid = (_lo + _hi) / 2;
	_mid = _mid - (_mid mod 1);
	_mid = _to_sort select _mid;

	// Compare
	while {_i <= _j} do {
		while {
			(call loadFile Format ["\:STRING COMPARE %1 text1:%2text2:%3", _options, (_to_sort select _i), _mid]) == -1
		} do {
			_i = _i + 1;
		};
		
		while {
			(call loadFile Format ["\:STRING COMPARE %1 text1:%2text2:%3", _options, (_to_sort select _j), _mid]) ==  1
		} do {
			_j = _j - 1;
		};
					
		// Replace values in all passed arrays
		if (_i <= _j) then {
			{
				_temp = _x select _i;
				_x set [_i, _x select _j];
				_x set [_j, _temp];
			} forEach _input;
			
			_i = _i + 1;
			_j = _j - 1;
		}
	};

	// Recurrence
	if (_lo < _j) then {[_input, _lo, _j, _options] call quicksortABC_loop};
	if (_i < _hi) then {[_input, _i, _hi, _options] call quicksortABC_loop};	
};